#version 130

#include "settings.glsl"

varying vec2 TexCoords;
uniform sampler2D colortex4;
uniform float frameTimeCounter;

float ab = CHROMATIC_ABERRATION;
float zoom = ZOOM;
float strength = DIST_STRENGTH;
float vignetteStrength = VIGNETTE_STRENGTH;
float vignetteRadius = VIGNETTE_RADIUS;
float stripeWidth = BLACK_STRIPES_WIDTH;
float stripeEdgeSoftness = BLACK_STRIPES_SOFT;

vec2 cameraShake(float frameTimeCounter, float intensity) {
    return vec2(
        intensity * 0, // Horizontal shake
        intensity * cos(frameTimeCounter * 1.85)  // Vertical shake
    );
}

void main() {
	vec3 color = texture2DLod(colortex4, TexCoords, 0).rgb;

    // Apply zoom
    vec2 centeredCoord = (TexCoords - 0.5) * zoom + 0.5;
    float distance = length(centeredCoord - 0.5);

    // Camera shake effect
	vec2 shake = cameraShake(frameTimeCounter, INTENSITY_CAM_SHAKE);

    // Fisheye effect
    vec2 fisheyeUV = (centeredCoord + shake - 0.5) * (1.0 + strength * distance * distance) + 0.5;

	// Chromatic aberration
    vec2 redUV = fisheyeUV + vec2(ab, ab) * distance;
    vec2 greenUV = fisheyeUV;
    vec2 blueUV = fisheyeUV + vec2(-ab, -ab) * distance;
    color.r = texture2D(colortex4, redUV).r;
    color.g = texture2D(colortex4, greenUV).g;
    color.b = texture2D(colortex4, blueUV).b;


    #if SCANLINE == 1
    // Apply scanline effect
    float scanline = sin(fisheyeUV.y * SCANLINE_WIDTH * 1.5) * SCANLINE_STRENGTH;
    color.rgb += scanline;
    #endif

    #if GRAIN == 1
    // Apply noise grain
    float noise = (fract(sin(dot(fisheyeUV, vec2(12.9898, 78.233))) * 43758.5453) - 0.5) * GRAIN_STRENGTH;
    color.rgb += noise;
    #endif

    #if COLOR_DIST == 1
    // Color distortion
    float colorDistort = COLOR_DIST_STRENGTH * sin(frameTimeCounter * 2.0);
    color.rgb *= vec3(1.0 + colorDistort, 1.0 - colorDistort, 1.0 + colorDistort);
    #endif

    // Apply saturation, brightness, and contrast
    float average = (color.r + color.g + color.b) / 3.0;
    color.rgb = mix(vec3(average), color.rgb, SATURATION);
    color.rgb += BRIGHTNESS;
    color.rgb = ((color.rgb - 0.5) * CONTRAST) + 0.5;

    #if VIGNETTE == 1
    float vignette = smoothstep(vignetteRadius, vignetteRadius + vignetteStrength, distance);
    color = mix(color, vec3(0.0), vignette);
    #endif

    #if BLACK_STRIPES == 1
    float leftStripe = smoothstep(stripeWidth, stripeWidth - stripeEdgeSoftness, TexCoords.x);
    float rightStripe = smoothstep(1.0 - stripeWidth, 1.0 - (stripeWidth - stripeEdgeSoftness), TexCoords.x);
    float stripeEffect = max(leftStripe, rightStripe);
    color = mix(color, vec3(0.0), stripeEffect);
	#endif

	gl_FragColor = vec4(color, 1.0);
}
